#include "entity_health.hpp"
#include "unit.hpp"
#include "utils/godot_macros.hpp"

void EntityHealth::_bind_methods() {
#define CLASSNAME EntityHealth
    GDPROPERTY(max_health, gd::Variant::INT);

    gd::PropertyInfo const source_arg{};
    GDSIGNAL("damage",
            gd::PropertyInfo(gd::Variant::INT, "remaining"),
            gd::PropertyInfo(gd::Variant::INT, "change"),
            gd::PropertyInfo(gd::Variant::OBJECT, "source", gd::PROPERTY_HINT_NODE_TYPE, "Unit"));
    GDSIGNAL("heal",
            gd::PropertyInfo(gd::Variant::INT, "remaining"),
            gd::PropertyInfo(gd::Variant::INT, "change"),
            gd::PropertyInfo(gd::Variant::OBJECT, "source", gd::PROPERTY_HINT_NODE_TYPE, "Unit"));
    GDSIGNAL("health_changed",
            gd::PropertyInfo(gd::Variant::INT, "remaining"),
            gd::PropertyInfo(gd::Variant::INT, "change"));
    GDSIGNAL("death", gd::PropertyInfo(gd::Variant::OBJECT, "source", gd::PROPERTY_HINT_NODE_TYPE, "Unit"));
}

void EntityHealth::_enter_tree() {
    this->current_health = this->max_health;
    this->emit_signal("damage", this->current_health, 0, nullptr);
}

void EntityHealth::damaged_by(int amount, Unit *source) {
    amount = gd::Math::abs(amount);
    this->current_health -= amount;
    this->emit_signal("damage", this->current_health, amount, source);
    this->emit_signal("health_changed", this->current_health, -amount);
    if(this->current_health <= 0) {
        this->emit_signal("death", source);
    }
}

void EntityHealth::healed_by(int amount, Unit *source) {
    amount = gd::Math::abs(amount);
    this->current_health = gd::Math::min(this->max_health, this->current_health + amount);
    this->emit_signal("heal", this->current_health, amount, source);
    this->emit_signal("health_changed", this->current_health, amount);
}

void EntityHealth::set_max_health(int max_health) {
    this->max_health = max_health;
}

int EntityHealth::get_max_health() const {
    return this->max_health;
}

void EntityHealth::set_current_health(int current_health) {
    this->current_health = current_health;
}

int EntityHealth::get_current_health() const {
    return this->current_health;
}
